<?php

namespace seraph_accel;

if( !defined( 'ABSPATH' ) )
	exit;

class ImgSrc
{
	public $src;
	public $srcInfo;
	public $mimeType;

	function __construct( $src, $srcInfo = null )
	{
		$this -> src = $src;
		$this -> srcInfo = $srcInfo;

	}

	function Init( $ctxProcess, $requestDomainUrl = null, $requestUriPath = null )
	{
		if( !isset( $this -> srcInfo ) )
			$this -> srcInfo = Ui::IsSrcAttrData( $this -> src ) ? false : GetSrcAttrInfo( $ctxProcess, $requestDomainUrl, $requestUriPath, $this -> src );
	}

	function GetSize()
	{
		if( !isset( $this -> cont ) && $this -> srcInfo )
		{
			$srcRealFile = (isset($this -> srcInfo[ 'filePath' ])?$this -> srcInfo[ 'filePath' ]:null);
			if( $srcRealFile )
				return( @filesize( $srcRealFile ) );
		}

		$this -> GetCont();
		return( $this -> cont !== false ? strlen( $this -> cont ) : false );
	}

	function GetCont()
	{
		if( !isset( $this -> cont ) )
		{
			if( $this -> srcInfo )
			{
				$srcRealFile = (isset($this -> srcInfo[ 'filePath' ])?$this -> srcInfo[ 'filePath' ]:null);
				if( $srcRealFile )
					$this -> cont = @file_get_contents( $srcRealFile );
				else
					$this -> cont = GetExtContents( $this -> srcInfo[ 'url' ], $this -> mimeType, true, 10 );
			}
			else
			{
				$data = strpos( $this -> src, ',' );
				if( $data !== false )
				{
					$prms = explode( ';', substr( $this -> src, 5, $data - 5 ) );
					$data = substr( $this -> src, $data + 1 );
					$this -> mimeType = (isset($prms[ 0 ])?$prms[ 0 ]:null);
					$this -> cont = ( (isset($prms[ 1 ])?$prms[ 1 ]:null) == 'base64' ) ? base64_decode( $data ) : false;
				}
				else
					$this -> cont = false;
			}

			if( !$this -> mimeType )
			{
				if( $this -> cont && preg_match( '@<svg[>\\s]@i', $this -> cont ) && preg_match( '@</svg>@i', $this -> cont ) )
				{
					$this -> mimeType = 'image/svg+xml';
					$this -> info = false;
				}
				else
				{
					$this -> GetInfo();
					if( $this -> info )
						$this -> mimeType = $this -> info[ 'mime' ];
				}
			}
		}

		return( $this -> cont );
	}

	function GetInfo()
	{
		$this -> GetCont();

		if( !isset( $this -> info ) )
		{
			$this -> info = Img::GetInfoFromData( $this -> cont );
			if( $this -> info === null )
				$this -> info = false;
		}

		return( $this -> info );
	}
}

class ImgSzAlternatives
{
	const MIN			= 0;
	const MAX			= 999999999;

	public $cxMin;
	public $cxMax;
	public $a;

	function __construct( $cxMin = ImgSzAlternatives::MIN, $cxMax = ImgSzAlternatives::MAX )
	{
		$this -> cxMin = $cxMin;
		$this -> cxMax = $cxMax;
		$this -> a = array();
	}
}

function _Images_ProcessSrc_InlineSmallEx( $mimeType, $imgCont )
{

	return( 'data:' . $mimeType . ';base64,' . base64_encode( $imgCont ) );
}

function _Images_ProcessSrc_InlineSmall( $imgSrc, $settImg )
{
	if( !Gen::GetArrField( $settImg, array( 'inlSml' ), false ) )
		return( false );

	$fileSize = $imgSrc -> GetSize();
	if( $fileSize === false )
		return( false );

	if( !$fileSize || $fileSize > Gen::GetArrField( $settImg, array( 'inlSmlSize' ), 0 ) )
		return( false );

	$imgCont = $imgSrc -> GetCont();
	if( $imgCont === false )
		return( false );

	if( !$imgSrc -> mimeType )
		return( false );

	$imgSrc -> src = _Images_ProcessSrc_InlineSmallEx( $imgSrc -> mimeType, $imgCont );
	return( true );
}

function _FileWriteTmpAndReplace( $fileName, $fileTime, $data )
{
	$fileNameTmp = $fileName . '.tmp';

	$hr = Gen::FilePutContentExclusive( $fileNameTmp, $data );
	if( Gen::HrSucc( $hr ) )
	{
		@touch( $fileNameTmp, $fileTime );
		@rename( $fileNameTmp, $fileName );
	}
	else if( $hr != Gen::E_BUSY )
	{

		@unlink( $fileNameTmp );
		@unlink( $fileName );
	}
}

function _Images_ProcessSrc_ConvertEx( $type, $settImg, $imgSrc, $srcRealFile, $srcRealCnvFile, $fileTime, $fileTimeCnv )
{

	$srcRealCnvStatFile = $srcRealCnvFile . '.json';
	$fileTimeCnvStat = @filemtime( $srcRealCnvStatFile );

	$fileExt = Gen::GetFileExt( $srcRealFile );
	if( !( Gen::GetArrField( $settImg, array( $type, 'enable' ), false ) && in_array( $fileExt, array( 'jpe', 'jpg', 'jpeg', 'png', 'gif', 'bmp' ) ) && ( ( $fileTimeCnv === false && $fileTimeCnvStat === false ) || ( $fileTimeCnv !== false && $fileTime > $fileTimeCnv ) || ( $fileTimeCnvStat !== false && $fileTime > $fileTimeCnvStat ) ) ) )
		return;

	$data = @is_a( $imgSrc, 'seraph_accel\\ImgSrc' ) ? $imgSrc -> GetCont() : $imgSrc;

	$status = null;
	if( ( $fileExt == 'png' && Img::IsDataPngAnimated( $data ) ) || ( $fileExt == 'gif' && Img::IsDataGifAnimated( $data ) ) )
		$status = 'aniNotSupp';

	$dataCnv = null;
	if( !$status )
		$dataCnv = Img::ConvertData( $data, 'image/' . $type );

	$fileTime += Gen::GetArrField( array( 'webp' => 1, 'avif' => 2 ), array( $type ), 0 );

	if( $dataCnv )
	{
		if( strlen( $dataCnv ) < strlen( $data ) )
		{
			_FileWriteTmpAndReplace( $srcRealCnvFile, $fileTime, $dataCnv );
			@unlink( $srcRealCnvStatFile );
		}
		else
			$status = 'larger';
	}

	if( !$dataCnv || $status )
	{
		@unlink( $srcRealCnvFile );
		if( $status )
			_FileWriteTmpAndReplace( $srcRealCnvStatFile, $fileTime, @json_encode( $status ) );
		else
			@unlink( $srcRealCnvStatFile );
	}
}

function _Images_ProcessSrc_Convert( $type, $settImg, $imgSrc, $srcRealFile )
{
	$fileTime = @filemtime( $srcRealFile );
	$srcRealCnvFile = $srcRealFile . '.' . $type;
	$fileTimeWebp = @filemtime( $srcRealCnvFile );

	return( _Images_ProcessSrc_ConvertEx( $type, $settImg, $imgSrc, $srcRealFile, $srcRealCnvFile, $fileTime, $fileTimeWebp ) );
}

function Images_ProcessSrcEx( &$ctxProcess, $imgSrc, $settCache, $settImg )
{
	$args = $imgSrc -> srcInfo[ 'args' ];

	$srcRealFile = (isset($imgSrc -> srcInfo[ 'filePath' ])?$imgSrc -> srcInfo[ 'filePath' ]:null);

	if( !$srcRealFile )
	{
		$cache = false;
		foreach( Gen::GetArrField( $settImg, array( 'cacheExt' ), array() ) as $srcPattern )
		{
			if( !@preg_match( $srcPattern, $imgSrc -> src ) )
				continue;

			$cache = true;
			break;
		}

		if( !$cache )
			return( false );

		$imgCont = $imgSrc -> GetCont();
		if( $imgCont === false || !$imgSrc -> mimeType )
			return( false );

		if( Gen::GetArrField( $settImg, array( 'inlSml' ), false ) && strlen( $imgCont ) <= Gen::GetArrField( $settImg, array( 'inlSmlSize' ), 0 ) )
			$imgSrc -> src = _Images_ProcessSrc_InlineSmallEx( $imgSrc -> mimeType, $imgCont );
		else
		{
			if( !adkxsshiujqtfk( $ctxProcess, $settCache, array( 'img', Gen::GetFileName( $imgSrc -> mimeType ) ), $imgCont, $imgSrc -> src, $srcRealFile ) )
				return( false );

			foreach( array( 'webp', 'avif' ) as $typeCnv )
				_Images_ProcessSrc_Convert( $typeCnv, $settImg, $imgCont, $srcRealFile );
		}

		return( true );
	}

	$fileTime = @filemtime( $srcRealFile );
	if( !$fileTime )
		return( false );

	$fileTimeOrig = $fileTime;

	$srcRealWebpFile = $srcRealFile . '.webp';
	$fileTimeWebp = @filemtime( $srcRealWebpFile );

	$srcRealAvifFile = $srcRealFile . '.avif';
	$fileTimeAvif = @filemtime( $srcRealAvifFile );

	if( $fileTimeWebp !== false && $fileTimeWebp > $fileTime )
		$fileTime = $fileTimeWebp;
	if( $fileTimeAvif !== false && $fileTimeAvif > $fileTime )
		$fileTime = $fileTimeAvif;

	if( ( (isset($ctxProcess[ 'compatView' ])?$ctxProcess[ 'compatView' ]:null) !== 'cm' ) && _Images_ProcessSrc_InlineSmall( $imgSrc, $settImg ) )
		return( true );

	if( Gen::GetArrField( $settImg, array( 'srcAddLm' ), false ) )
	{
		$args[ 'lm' ] = sprintf( '%X', $fileTime );
		$imgSrc -> src = Net::UrlDeParse( array( 'path' => $imgSrc -> srcInfo[ 'srcWoArgs' ], 'query' => $args, 'fragment' => (isset($imgSrc -> srcInfo[ '#' ])?$imgSrc -> srcInfo[ '#' ]:null) ), Net::URLPARSE_F_PRESERVEEMPTIES );

	}

	_Images_ProcessSrc_ConvertEx( 'webp', $settImg, $imgSrc, $srcRealFile, $srcRealWebpFile, $fileTimeOrig, $fileTimeWebp );
	_Images_ProcessSrc_ConvertEx( 'avif', $settImg, $imgSrc, $srcRealFile, $srcRealAvifFile, $fileTimeOrig, $fileTimeAvif );

	return( true );
}

function Images_ProcessSrcSizeAlternatives( $imgSzAlternatives, &$ctxProcess, $imgSrc, $settCache, $settImg, $settCdn )
{
	$imgSrc -> Init( $ctxProcess );

	$info = $imgSrc -> GetInfo();
	if( !$info )
		return;

	$img = null;

	foreach( array( array( 1366, 1.2 ), array( 768, 1.1 ), array( 480, 1.0 ), array( 360, 1.0 ) ) as $cx )
	{
		$scale = ( float )$info[ 'cx' ] / $cx[ 0 ];
		if( $scale < $cx[ 1 ] || !( $imgSzAlternatives -> cxMin <= $cx[ 0 ] && $cx[ 0 ] <= $imgSzAlternatives -> cxMax ) || !( Gen::GetArrField( $settImg, array( 'szAdaptBgCxMin' ), 0 ) <= $cx[ 0 ] ) )
			continue;

		$cx = $cx[ 0 ];
		$cy = ( int )round( ( float )$info[ 'cy' ] * ( $cx / $info[ 'cx' ] ) );

		if( $img === null )
		{
			$img = Img::CreateFromData( $imgSrc -> GetCont() );
			if( !$img )
				return;
		}

		$imgNew = Img::CreateCopyResample( $img,
			array( 'cx' => $cx, 'cy' => $cy ),
			array( 'x' => 0, 'y' => 0, 'cx' => $info[ 'cx' ], 'cy' => $info[ 'cy' ] ) );
		if( !$imgNew )
			continue;

		$imgNewCont = Img::GetData( $imgNew, $info[ 'mime' ] );
		imagedestroy( $imgNew );
		if( !$imgNewCont )
			continue;

		$fileType = Gen::GetFileName( $info[ 'mime' ] );

		if( Gen::GetArrField( $settImg, array( 'inlSml' ), false ) && $info[ 'mime' ] && strlen( $imgNewCont ) <= Gen::GetArrField( $settImg, array( 'inlSmlSize' ), 0 ) )
			$imgSrcAlter = _Images_ProcessSrc_InlineSmallEx( $info[ 'mime' ], $imgNewCont );
		else
		{
			if( !adkxsshiujqtfk( $ctxProcess, $settCache, array( 'img', $fileType ), $imgNewCont, $imgSrcAlter, $filePath ) )
				continue;

			foreach( array( 'webp', 'avif' ) as $typeCnv )
				_Images_ProcessSrc_Convert( $typeCnv, $settImg, $imgNewCont, $filePath );
		}

		Cdn_AdjustUrl( $ctxProcess, $settCdn, $imgSrcAlter, $fileType );
		Fullness_AdjustUrl( $ctxProcess, $imgSrcAlter );

		$imgSzAlternatives -> a[ ( string )$cx ] = $imgSrcAlter;
	}

	if( $img )
		imagedestroy( $img );

}

function Images_ProcessSrc( &$ctxProcess, $imgSrc, $settCache, $settImg, $settCdn )
{
	if( !$imgSrc -> src )
		return( false );

	$imgSrc -> Init( $ctxProcess );
	if( !$imgSrc -> srcInfo )
		return( false );

	$fileType = strtolower( Gen::GetFileExt( (isset($imgSrc -> srcInfo[ 'srcWoArgs' ])?$imgSrc -> srcInfo[ 'srcWoArgs' ]:null) ) );

	$adjusted = false;

	if( Images_ProcessSrcEx( $ctxProcess, $imgSrc, $settCache, $settImg ) )
		$adjusted = true;

	if( Cdn_AdjustUrl( $ctxProcess, $settCdn, $imgSrc -> src, $fileType ) )
		$adjusted = true;
	if( Fullness_AdjustUrl( $ctxProcess, $imgSrc -> src, (isset($imgSrc -> srcInfo[ 'srcUrlFullness' ])?$imgSrc -> srcInfo[ 'srcUrlFullness' ]:null) ) )
		$adjusted = true;

	return( $adjusted );
}

function Images_ProcessSrcSet( &$ctxProcess, &$srcset, $settCache, $settImg, $settCdn )
{
	$apply = false;

	$srcItems = Ui::ParseSrcSetAttr( $srcset );
	foreach( $srcItems as &$srcItem )
	{
		$imgSrc = new ImgSrc( html_entity_decode( $srcItem[ 0 ] ) );
		if( Images_ProcessSrc( $ctxProcess, $imgSrc, $settCache, $settImg, $settCdn ) )
		{
			$srcItem[ 0 ] = $imgSrc -> src;
			$apply = true;
		}
	}

	if( !$apply )
		return( false );

	$srcset = Ui::GetSrcSetAttr( $srcItems, false );
	return( true );
}

function LazyLoad_SrcSubst( $width, $height, $exact = false )
{
	if( !$width )
		$width = 225;
	if( !$height )
		$height = $width / 3 * 2;
	return( 'data:image/svg+xml,' . rawurlencode( Ui::Tag( 'svg',

		null

		, array_merge( array( 'xmlns' => 'http://www.w3.org/2000/svg', 'viewBox' => '0 0 ' . $width . ' ' . $height ), $exact ? array( 'width' => $width, 'height' => $height ) : array() ) ) ) );

}

function _Images_ProcessItemLazy( &$ctxProcess, $doc, $settImg, $item, $imgSrc )
{
	if( $ctxProcess[ 'isAMP' ] || !Gen::GetArrField( $settImg, array( 'lazy', 'load' ), false ) )
		return( false );

	if( HtmlNd::FindUpByTag( $item, 'noscript' ) )
		return( false );

	if( $ctxProcess[ 'imgLazySkipCount' ] )
	{
		$ctxProcess[ 'imgLazySkipCount' ]--;
		return( false );
	}

	if( Gen::GetArrField( $settImg, array( 'lazy', 'del3rd' ), false ) )
	{
		if( $item -> getAttribute( 'loading' ) == 'lazy' )
			$item -> removeAttribute( 'loading' );
		HtmlNd::RenameAttr( $item, 'data-src', 'src' );
		HtmlNd::RenameAttr( $item, 'data-srcset', 'srcset' );
	}

	$src = $item -> getAttribute( 'src' );
	if( !$src )
		return( false );
	if( !$item -> getAttribute( 'srcset' ) && Ui::IsSrcAttrData( $src ) )
		return( false );

	if( Images_CheckLazyExcl( $ctxProcess, $doc, $settImg, $item ) )
		return( false );

	{
		$itemCopy = $item -> cloneNode( true );
		if( !$itemCopy )
			return( false );

		$itemNoScript = $doc -> createElement( 'noscript' );
		if( !$itemNoScript )
			return( false );

		$itemNoScript -> setAttribute( 'lzl', '' );
		$itemNoScript -> appendChild( $itemCopy );
		HtmlNd::InsertAfter( $item -> parentNode, $itemNoScript, $item );
	}

	$ctxProcess[ 'lazyload' ] = true;
	HtmlNd::AddAttrClass( $item, array( 'lzl' ) );

	$srcImgDim = $imgSrc ? $imgSrc -> GetInfo() : null;

	$width = $srcImgDim ? $srcImgDim[ 'cx' ] : 225;
	$height = $srcImgDim ? $srcImgDim[ 'cy' ] : $width / 3 * 2;

	HtmlNd::RenameAttr( $item, 'srcset', 'data-lzl-srcset' );
	HtmlNd::RenameAttr( $item, 'sizes', 'data-lzl-sizes' );

	$item -> setAttribute( 'data-lzl-src', $src );

	$item -> setAttribute( 'src', LazyLoad_SrcSubst( $width, $height, true ) );

	{
		for( $p = $item -> parentNode; $p && $p -> nodeType == XML_ELEMENT_NODE; $p = $p -> parentNode )
		{
			if( !in_array( 'woocommerce-product-gallery', Ui::ParseClassAttr( $p -> getAttribute( 'class' ) ) ) )
				continue;

			$styles = Ui::ParseStyleAttr( $p -> getAttribute( 'style' ) );
			$styles[ 'opacity' ] = 1;

			$p -> setAttribute( 'style', Ui::GetStyleAttr( $styles ) );
			break;
		}
	}

	return( true );
}

function Images_ProcessItemLazyBg( &$ctxProcess, $doc, $settImg, $item, $imgSrc )
{
	if( HtmlNd::FindUpByTag( $item, 'noscript' ) )
		return( false );

	if( $ctxProcess[ 'imgLazySkipCount' ] )
	{
		$ctxProcess[ 'imgLazySkipCount' ]--;
		return( false );
	}

	if( $item -> hasAttribute( 'data-bg' ) )
		return( false );

	if( Images_CheckLazyExcl( $ctxProcess, $doc, $settImg, $item ) )
		return( false );

	$ctxProcess[ 'lazyload' ] = true;
	HtmlNd::AddAttrClass( $item, array( 'lzl' ) );

	$item -> setAttribute( 'data-lzl-bg', $imgSrc -> src );

	$srcImgDim = $imgSrc -> GetInfo();
	$width = $srcImgDim ? $srcImgDim[ 'cx' ] : 225;
	$height = $srcImgDim ? $srcImgDim[ 'cy' ] : $width / 3 * 2;
	$imgSrc -> src = LazyLoad_SrcSubst( $width, $height, true );

	return( true );
}

function Images_CheckLazyExcl( &$ctxProcess, $doc, $settImg, $item )
{
	$lazyExclItems = &$ctxProcess[ 'lazyExclItems' ];
	if( $lazyExclItems === null )
	{
		$lazyExclItems = array();

		$excls = Gen::GetArrField( $settImg, array( 'lazy', 'excl' ), array() );
		if( $excls )
		{
			$xpath = new \DOMXPath( $doc );

			foreach( $excls as $exclItemPath )
				foreach( HtmlNd::ChildrenAsArr( @$xpath -> query( $exclItemPath, $ctxProcess[ 'ndHtml' ] ) ) as $itemExcl )
					$lazyExclItems[] = $itemExcl;
		}
	}

	return( in_array( $item, $lazyExclItems, true ) );
}

function Images_CheckSzAdaptExcl( &$ctxProcess, $doc, $settImg, $item )
{
	$excls = Gen::GetArrField( $settImg, array( 'szAdaptExcl' ), array() );
	if( !$excls )
		return( false );

	$ctxSzAdaptExcl = (isset($ctxProcess[ 'ctxSzAdaptExcl' ])?$ctxProcess[ 'ctxSzAdaptExcl' ]:null);
	if( !$ctxSzAdaptExcl )
		$ctxProcess[ 'ctxSzAdaptExcl' ] = $ctxSzAdaptExcl = new AnyObj();

	$itemRoot = $ctxProcess[ 'ndHtml' ];
	if( is_string( $item ) )
	{
		if( !(isset($ctxSzAdaptExcl -> itemTmp)?$ctxSzAdaptExcl -> itemTmp:null) )
		{
			$ctxSzAdaptExcl -> itemTmpCont = $doc -> createElement( 'root' );
			$ctxSzAdaptExcl -> itemTmpCont -> appendChild( $ctxSzAdaptExcl -> itemTmp = $doc -> createElement( 'style' ) );
		}

		HtmlNd::SetValFromContent( $ctxSzAdaptExcl -> itemTmp, $item );
		$item = $ctxSzAdaptExcl -> itemTmp;
		$itemRoot = $ctxSzAdaptExcl -> itemTmpCont;
	}

	$xpath = new \DOMXPath( $doc );

	$found = false;
	foreach( $excls as $exclItem )
	{
		$items = HtmlNd::ChildrenAsArr( @$xpath -> query( $exclItem, $itemRoot ) );
		if( in_array( $item, $items, true ) )
		{
			$found = true;
			break;
		}
	}

	HtmlNd::SetValFromContent( (isset($ctxSzAdaptExcl -> itemTmp)?$ctxSzAdaptExcl -> itemTmp:null), '' );
	return( $found );
}

function Images_Process( &$ctxProcess, $doc, $settCache, $settImg, $settCdn )
{
	if( !( Gen::GetArrField( $settImg, array( 'srcAddLm' ), false ) || Gen::GetArrField( $settImg, array( 'inlSml' ), false ) || Gen::GetArrField( $settImg, array( 'lazy', 'setSize' ), false ) || Gen::GetArrField( $settImg, array( 'lazy', 'load' ), false ) || Gen::GetArrField( $settCdn, array( 'enable' ), false ) ) )
		return( true );

	$ctxProcess[ 'imgLazySkipCount' ] = 0;

	$items = HtmlNd::ChildrenAsArr( $doc -> getElementsByTagName( 'img' ) );
	if( $ctxProcess[ 'isAMP' ] )
		$items = array_merge( $items, HtmlNd::ChildrenAsArr( $doc -> getElementsByTagName( 'amp-img' ) ) );
	foreach( $items as $item )
	{
		if( ContentProcess_IsAborted() ) return( true );

		if( !$item -> attributes )
			continue;

		$inlinedSize = 0;
		$imgSrc = null;

		$attr = $item -> attributes -> getNamedItem( 'src' );
		if( $attr )
		{
			$imgSrc = new ImgSrc( html_entity_decode( $attr -> nodeValue ) );
			if( Images_ProcessSrc( $ctxProcess, $imgSrc, $settCache, $settImg, $settCdn ) )
			{
				$attr -> nodeValue = htmlspecialchars( $imgSrc -> src );

				if( Gen::GetArrField( $settImg, array( 'lazy', 'setSize' ), false ) && !$item -> hasAttribute( 'width' ) && !$item -> hasAttribute( 'height' ) && ( $srcImgDim = $imgSrc -> GetInfo() ) )
				{
					$item -> setAttribute( 'width', $srcImgDim[ 'cx' ] );
					$item -> setAttribute( 'height', $srcImgDim[ 'cy' ] );
				}
			}

			if( Ui::IsSrcAttrData( $imgSrc -> src ) )
				$inlinedSize = strlen( $imgSrc -> src );
		}

		if( $attrSrcSet = $item -> attributes -> getNamedItem( 'srcset' ) )
		{
			$srcset = $attrSrcSet -> nodeValue;
			if( Images_ProcessSrcSet( $ctxProcess, $srcset, $settCache, $settImg, $settCdn ) )
				$attrSrcSet -> nodeValue = htmlspecialchars( $srcset );

			if( stripos( $srcset, 'data:' ) !== false )
				$inlinedSize = strlen( $srcset );
		}

		if( $inlinedSize >= 2048 && (isset($ctxProcess[ 'chunksEnabled' ])?$ctxProcess[ 'chunksEnabled' ]:null) )
			ContentMarkSeparate( $item );

		_Images_ProcessItemLazy( $ctxProcess, $doc, $settImg, $item, $imgSrc );
	}

	foreach( HtmlNd::ChildrenAsArr( $doc -> getElementsByTagName( 'picture' ) ) as $itemPict )
	{
		if( ContentProcess_IsAborted() ) return( true );

		foreach( $itemPict -> childNodes as $item )
		{
			if( $item -> nodeName != 'source' )
				continue;

			if( !$item -> attributes )
				continue;

			$attrSrcSet = $item -> attributes -> getNamedItem( 'srcset' );
			if( $attrSrcSet )
			{
				$srcset = $attrSrcSet -> nodeValue;
				if( Images_ProcessSrcSet( $ctxProcess, $srcset, $settCache, $settImg, $settCdn ) )
					$attrSrcSet -> nodeValue = htmlspecialchars( $srcset );
			}

			if( Gen::GetArrField( $settImg, array( 'lazy', 'load' ), false ) )
			{

				{
					$itemCopy = $item -> cloneNode( true );
					if( !$itemCopy )
						return( false );

					$itemNoScript = $doc -> createElement( 'noscript' );
					if( !$itemNoScript )
						return( false );

					$itemNoScript -> setAttribute( 'lzl', '' );
					$itemNoScript -> appendChild( $itemCopy );
					HtmlNd::InsertAfter( $item -> parentNode, $itemNoScript, $item );
				}

				$ctxProcess[ 'lazyload' ] = true;
				HtmlNd::RenameAttr( $item, 'srcset', 'data-lzl-srcset' );
			}
		}
	}

	if( Gen::GetArrField( $settImg, array( 'srcAddLm' ), false ) || Gen::GetArrField( $settCdn, array( 'enable' ), false ) )
	{
		$srcImgDim = null;

		$settImgForMeta = Gen::ArrCopy( $settImg );
		Gen::SetArrField( $settImgForMeta, array( 'inlSml' ), false );

		foreach( $ctxProcess[ 'ndHead' ] -> childNodes as $item )
		{
			if( ContentProcess_IsAborted() ) return( true );

			$srcAttrName = null; $src = null;

			if( $item -> nodeName == 'meta' )
			{
				$id = $item -> getAttribute( 'property' );
				if( !$id )
					$id = $item -> getAttribute( 'name' );

				if( $id && in_array( $id, array( 'og:image', 'og:image:secure_url', 'twitter:image', 'vk:image' ) ) )
					$srcAttrName = 'content';
			}
			else if( $item -> nodeName == 'link' )
			{
				if( $item -> getAttribute( 'rel' ) == 'icon' )
					$srcAttrName = 'href';
			}

			if( !$srcAttrName )
				continue;

			$src = $item -> getAttribute( $srcAttrName );
			if( !$src )
				continue;

			$src = new ImgSrc( $src );
			if( Images_ProcessSrc( $ctxProcess, $src, $settCache, $settImgForMeta, $settCdn ) )
				$item -> setAttribute( $srcAttrName, $src -> src );
		}
	}

	return( true );
}

