<?php

namespace seraph_accel;

if( !defined( 'ABSPATH' ) )
	exit;

function _CacheDirWalk_User( $path, $item, &$ctx )
{
	$path = $path . '/' . $item . '/s';
	if( !@is_dir( $path ) )
		return( true );

	return( Gen::DirEnum( $path, $ctx,
		function( $path, $item, &$ctx )
		{
			$path = $path . '/' . $item . '/s';
			if( !@is_dir( $path ) )
				return( true );

			$ctx[ 'isDefSess' ] = ( $item == '@' );

			return( Gen::DirEnum( $path, $ctx,
				function( $path, $item, &$ctx )
				{
					$path = $path . '/' . $item . '/o';
					if( !@is_dir( $path ) )
						return( true );

					$ctx[ 'isDefState' ] = ( $item == '@' );
					$ctx[ 'dirRootLen' ] = strlen( $path );

					$recurse = true;
					$objPathSpec = Gen::GetArrField( $ctx[ 'spec' ], array( 'objPath' ) );
					if( $objPathSpec !== null )
					{
						if( $objPathSpec )
							$path .= '/' . $objPathSpec;
						$recurse = Gen::GetArrField( $ctx[ 'spec' ], array( 'objPathRecurse' ), false );
					}

					return( Gen::DirEnum( $path, $ctx,
						function( $path, $item, &$ctx )
						{
							$item = $path . '/' . $item;
							$isUserCtx = !( $ctx[ 'isDefSess' ] && $ctx[ 'isDefState' ] );
							if( @is_dir( $item ) )
								return( $ctx[ 'cbPath' ] ? call_user_func_array( $ctx[ 'cbPath' ], array( &$ctx[ 'ctxWalk' ], $isUserCtx, str_replace( '\\', '/', substr( $item, $ctx[ 'dirRootLen' ] ) ) ) ) : true );
							return( call_user_func_array( $ctx[ 'cbObj' ], array( &$ctx[ 'ctxWalk' ], $isUserCtx, $item ) ) );
						}
						, $recurse
					) );
				}
			) );
		}
	) );
}

function _CacheDirWalk_View( $path, $item, &$ctx )
{
	$path .= '/' . $item;
	if( !@is_dir( $path ) )
		return( true );

	$viewDir = $path;

	if( $cbView = $ctx[ 'cbView' ] )
		call_user_func_array( $cbView, array( &$ctx[ 'ctxWalk' ], $viewDir, true ) );

	$path .= '/c';
	if( !@is_dir( $path ) )
		return( true );

	$userIdSpec = Gen::GetArrField( $ctx[ 'spec' ], array( 'userId' ) );
	$res = $userIdSpec ? _CacheDirWalk_User( $path, '' . $userIdSpec, $ctx ) : Gen::DirEnum( $path, $ctx, 'seraph_accel\\_CacheDirWalk_User' );

	if( $cbView = $ctx[ 'cbView' ] )
		call_user_func_array( $cbView, array( &$ctx[ 'ctxWalk' ], $viewDir, false ) );

	return( $res );
}

function _CacheDirWalk( $siteId, $siteSubId, $viewId, &$ctxWalk, $spec = null, $cbObj = null, $cbData = null, $cbPath = null, $cbView = null, $cbSite = null )
{
	$siteCacheRootPath = GetCacheDir() . '/s/' . $siteId;

	$ctx = array( 'ctxWalk' => &$ctxWalk, 'spec' => $spec, 'cbObj' => $cbObj, 'cbPath' => $cbPath, 'cbData' => $cbData, 'cbView' => $cbView );

	if( $cbSite )
		if( call_user_func_array( $cbSite, array( &$ctx[ 'ctxWalk' ], $siteCacheRootPath, true ) ) === false )
			return( false );

	if( $cbObj )
	{
		foreach( ( $siteSubId === null ) ? @glob( GetCacheViewsDir( $siteCacheRootPath ) . '*' ) : array( GetCacheViewsDir( $siteCacheRootPath ) . '-' . $siteSubId ) as $viewDir )
		{
			if( ( $viewId === null ? Gen::DirEnum( $viewDir, $ctx, 'seraph_accel\\_CacheDirWalk_View' ) : _CacheDirWalk_View( $viewDir, $viewId, $ctx ) ) === false )
				return( false );
		}
	}

	if( $cbData )
	{
		$cacheDataDir = GetCacheDataDir( $siteCacheRootPath );
		$ctx[ 'cacheDataDirLen' ] = strlen( $cacheDataDir );

		if( Gen::DirEnum( $cacheDataDir, $ctx,
			function( $path, $item, &$ctx )
			{
				$itemId = $item;

				$item = $path . '/' . $item;
				if( @is_dir( $item ) )
					return( true );

				$itemId = explode( '.', $itemId );
				$itemId = '' . (isset($itemId[ 0 ])?$itemId[ 0 ]:null) . '.' . (isset($itemId[ 1 ])?$itemId[ 1 ]:null);

				$itemType = substr( $path, $ctx[ 'cacheDataDirLen' ] + 1 );
				return( call_user_func_array( $ctx[ 'cbData' ], array( &$ctx[ 'ctxWalk' ], $itemType ? $itemType : 'html', $itemId, $item ) ) );
			}
		, true ) === false )
		{
			return( false );
		}
	}

	if( $cbSite )
		if( call_user_func_array( $cbSite, array( &$ctx[ 'ctxWalk' ], $siteCacheRootPath, false ) ) === false )
			return( false );

	return( true );
}

function CacheGetInfo( $siteId, $cbCancel )
{
	$info = array( 'cbCancel' => $cbCancel, 'nObj' => 0, 'nJs' => 0, 'nCss' => 0, 'nImg' => 0, 'nLrn' => 0, 'size' => 0, 'sizeUncompr' => 0, 'sizeLrn' => 0, 'sizeObjFrag' => 0, 'sizeObj' => 0 );

	if( _CacheDirWalk( $siteId, null, null, $info, null,
		function( &$info, $isUserCtx, $objFile )
		{
			if( $info[ 'cbCancel' ]() )
				return( false );

			$info[ 'nObj' ] += 1;
			$sz = @filesize( $objFile );
			$info[ 'size' ] += $sz;
			$info[ 'sizeUncompr' ] += $sz;

			foreach( Gen::GetArrField( CacheReadDsc( $objFile ), array( 'p' ), array() ) as $oiCi )
			{
				if( $info[ 'cbCancel' ]() )
					return( false );

				$sz = GetCacheCos( $oiCi );

				$info[ 'sizeObj' ] += $sz;

			}
		}

		,
		function( &$info, $dataType, $dataId, $dataFile )
		{
			if( $info[ 'cbCancel' ]() )
				return( false );

			$sz = @filesize( $dataFile );
			$info[ 'size' ] += $sz;

			if( $dataType == 'img' )
			{
				$info[ 'nImg' ] += 1;
				$info[ 'sizeUncompr' ] += $sz;
			}
			else if( $dataType == Gen::GetFileExt( $dataFile ) )
			{
				switch( $dataType )
				{
				case 'js':		$info[ 'nJs' ] += 1; break;
				case 'css':		$info[ 'nCss' ] += 1; break;
				}

				$info[ 'sizeUncompr' ] += $sz;
			}
			else
				$info[ 'sizeUncompr' ] += GetCacheCos( Gen::GetFileName( Gen::GetFileName( $dataFile, true ), true ) );

			if( $dataType == 'html' )
			{
				$fileNameCount = null;
				if( $dataType == Gen::GetFileExt( $dataFile ) )
					$fileNameCount = $dataFile;
				else
				{
					$dataFileUncompr = Gen::GetFileName( $dataFile, true, true );
					if( $dataType == Gen::GetFileExt( $dataFileUncompr ) && !@file_exists( $dataFileUncompr ) )
						$fileNameCount = $dataFileUncompr;
				}

				if( $fileNameCount )
					$info[ 'sizeObjFrag' ] += GetCacheCos( Gen::GetFileName( $fileNameCount, true ) );
			}
		}

		,
		null

		,
		function( &$info, $viewDir, $begin )
		{
			if( !$begin )
				return;

			if( Gen::DirEnum( $viewDir . '/l', $info,
				function( $path, $item, &$info )
				{
					if( $info[ 'cbCancel' ]() )
						return( false );

					$path = $path . '/' . $item;
					if( @is_dir( $path ) )
						return;

					$sz = @filesize( $path );
					$info[ 'size' ] += $sz;
					$info[ 'sizeUncompr' ] += $sz;
					$info[ 'sizeLrn' ] += $sz;

					$info[ 'nLrn' ] += 1;
				}
			, true ) === false )
			{
				return( false );
			}
		}

		,
		function( &$info, $siteDir, $begin )
		{
			if( !$begin )
				return;

			if( Gen::DirEnum( $siteDir . '/l', $info,
				function( $path, $item, &$info )
				{
					if( $info[ 'cbCancel' ]() )
						return( false );

					$path = $path . '/' . $item;
					if( @is_dir( $path ) )
						return;

					$sz = @filesize( $path );
					$info[ 'size' ] += $sz;
					$info[ 'sizeUncompr' ] += GetCacheCos( Gen::GetFileName( Gen::GetFileName( $path, true ), true ) );
					$info[ 'sizeLrn' ] += $sz;
				}
			, true ) === false )
			{
				return( false );
			}
		}
	) === false )
	{
		return( null );
	}

	unset( $info[ 'cbCancel' ] );
	return( $info );
}

class CacheUrlProcessor
{
	private $viewsHeaders;
	private $priority;
	private $siteId;
	private $sitePathLen;
	private $settCache;

	public function __construct( $cbIsAbort, $siteId, $settCache, $priority, $viewId = null )
	{
		$this -> cbIsAbort = $cbIsAbort;
		$this -> siteId = $siteId;
		$this -> settCache = $settCache;
		$this -> viewsHeaders = CacheOpGetViewsHeaders( $settCache, $viewId );
		$this -> priority = $priority;
	}

	function __destruct()
	{
	}

	function isAbort()
	{
		return( $this -> cbIsAbort ? @call_user_func_array( $this -> cbIsAbort, array() ) : false );
	}

	function op( $siteAddr, $path, $query )
	{

		if( IsUriByPartsExcluded( $this -> settCache, $path, $query ) )
			return( null );

		$url = CacheOpUrl_DeParseUrl( $siteAddr, $path, $query );

		foreach( $this -> viewsHeaders as $headers )
		{
			if( $this -> isAbort() )
				return( false );

			if( CachePostPreparePageEx( $url, $this -> siteId, $this -> priority, $headers ) )
			{

			}
			usleep( 1 );
		}

		return( true );
	}
}

function CacheOp( $op, $priority = 0 )
{
	if( get_option( 'seraph_accel_cacheOp' ) !== false )
		return( false );

	_SetExpirableOption( 'seraph_accel_cacheOp', $op, 1800 );

	$curSiteId = GetSiteId();
	$optDropCache = new WpOptionNonAutoloadDropCache( 'seraph_accel_cacheOp', 5 );

	$ctx = array( 'op' => $op, 'datasDel' => array() );

	if( $op == 1 )
	{
		$sett = Plugin::SettGet();
		$ctx[ 'timeout' ] = Gen::GetArrField( $sett, array( 'cache', 'timeoutCln' ), 0 ) * 60;
		$ctx[ 'timeoutCtx' ] = Gen::GetArrField( $sett, array( 'cache', 'ctxTimeoutCln' ), 0 ) * 60;
		$ctx[ 'tmCur' ] = Gen::GetCurRequestTime();
		unset( $sett );

		if( _CacheDirWalk( $curSiteId, null, null, $ctx, null,
			null,

			function( &$ctx, $dataType, $dataId, $dataFile )
			{
				if( get_option( 'seraph_accel_cacheOp' ) === false )
					return( false );

				$ctx[ 'datasDel' ][ $dataType ][ $dataId ] = true;
			}
		) === false )
		{
			return( false );
		}
	}

	if( _CacheDirWalk( $curSiteId, null, null, $ctx, null,
		function( &$ctx, $isUserCtx, $objFile )
		{
			if( get_option( 'seraph_accel_cacheOp' ) === false )
				return( false );

			if( $ctx[ 'op' ] != 1 )
			{
				_CacheObjFileOp( $objFile, $ctx[ 'op' ] );
				return;
			}

			{
				$dscFileTm = @filemtime( $objFile );
				if( $dscFileTm > 0 )
				{
					$dscFileTmAge = $ctx[ 'tmCur' ] - $dscFileTm;
					$timeout = $isUserCtx ? $ctx[ 'timeoutCtx' ] : $ctx[ 'timeout' ];

					if( $timeout > 0 && $dscFileTmAge > $timeout )
					{
						@unlink( $objFile );
						return;
					}
				}
			}

			$dsc = CacheReadDsc( $objFile );
			if( !$dsc )
			{

				return;
			}

			if( isset( $dsc[ 'l' ] ) )
			{
				unset( $ctx[ 'lrnsDel' ][ $dsc[ 'l' ] ] );

				$lrnDsc = Learn_ReadDsc( $ctx[ 'curViewDir' ] . '/l/' . Learn_Id2File( $dsc[ 'l' ] ) );
				Learn_KeepNeededData( $ctx[ 'datasDel' ], $ctx[ 'lrnsGlobDel' ], $lrnDsc, $ctx[ 'lrnDataPath' ] );
			}

			$chunks = (isset($dsc[ 'p' ])?$dsc[ 'p' ]:null);
			if( !is_array( $chunks ) )
			{

				return;
			}

			foreach( $chunks as $oiCi )
				unset( $ctx[ 'datasDel' ][ 'html' ][ $oiCi ] );

			$childrenTypes = (isset($dsc[ 's' ])?$dsc[ 's' ]:null);
			if( $childrenTypes !== null )
			{
				if( !is_array( $childrenTypes ) )
				{

					return;
				}

				foreach( $childrenTypes as $childType => $children )
					foreach( $children as $childId )
						unset( $ctx[ 'datasDel' ][ $childType ][ $childId ] );
			}
		}

		, $op == 2 ? function( &$ctx, $dataType, $dataId, $dataFile )
		{
			if( get_option( 'seraph_accel_cacheOp' ) === false )
				return( false );

			@unlink( $dataFile );
		} : null

		,
		null

		,
		function( &$ctx, $viewDir, $begin )
		{
			if( $begin )
			{
				$ctx[ 'curViewDir' ] = $viewDir;
				if( $ctx[ 'op' ] == 2 )
					Gen::DelDir( $viewDir . '/l' );
			}

			if( $ctx[ 'op' ] != 1 )
				return;

			if( $begin )
			{
				$ctx[ 'lrnsDel' ] = array();

				if( Gen::DirEnum( $viewDir . '/l', $ctx,
					function( $path, $item, &$ctx )
					{
						if( get_option( 'seraph_accel_cacheOp' ) === false )
							return( false );

						if( @is_dir( $path . '/' . $item ) )
							return;

						$ctx[ 'lrnsDel' ][ Gen::GetFileName( $path ) . '/' . hex2bin( Gen::GetFileName( Gen::GetFileName( $item, true ), true ) ) ] = true;
					}
				, true ) === false )
				{
					return( false );
				}
			}
			else
			{
				foreach( $ctx[ 'lrnsDel' ] as $learnId => $del )
					Learn_Clear( $viewDir . '/l/' . Learn_Id2File( $learnId ) );
			}
		}

		,
		function( &$ctx, $siteDir, $begin )
		{
			if( $begin )
			{
				$ctx[ 'lrnDataPath' ] = $siteDir . '/l';
				$ctx[ 'lrnsGlobDel' ] = array();

				if( $ctx[ 'op' ] == 2 )
					Gen::DelDir( $ctx[ 'lrnDataPath' ] );

				if( $ctx[ 'op' ] != 1 )
					return;

				if( Gen::DirEnum( $ctx[ 'lrnDataPath' ], $ctx,
					function( $path, $item, &$ctx )
					{
						if( get_option( 'seraph_accel_cacheOp' ) === false )
							return( false );

						$path .= '/' . $item;
						if( !@is_dir( $path ) )
							$ctx[ 'lrnsGlobDel' ][ str_replace( '\\', '/', substr( $path, strlen( $ctx[ 'lrnDataPath' ] ) + 1 ) ) ] = true;
					}
				, true ) === false )
				{
					return( false );
				}
			}
			else
			{
				foreach( $ctx[ 'lrnsGlobDel' ] as $file => $del )
					@unlink( $siteDir . '/l/' . $file );
			}
		}
	) === false )
	{
		return( false );
	}

	if( $op == 1 )
	{
		foreach( $ctx[ 'datasDel' ] as $datasDelType => $datasDel )
			if( empty( $datasDel ) )
				unset( $ctx[ 'datasDel' ][ $datasDelType ] );

		if( $ctx[ 'datasDel' ] )
		{
			if( _CacheDirWalk( $curSiteId, null, null, $ctx, null,
				null,
				function( &$ctx, $dataType, $dataId, $dataFile )
				{
					if( get_option( 'seraph_accel_cacheOp' ) === false )
						return( false );

					if( (isset($ctx[ 'datasDel' ][ $dataType ][ $dataId ])?$ctx[ 'datasDel' ][ $dataType ][ $dataId ]:null) )
						@unlink( $dataFile );
				}
			) === false )
			{
				return( false );
			}
		}
	}

	if( $op == 1 || $op == 2 )
	{
		foreach( glob( GetCacheViewsDir( GetCacheDir() . '/s/' . $curSiteId ) . '*' ) as $viewDir )
		{
			if( Gen::DirEnum( $viewDir, $ctx,
				function( $path, $item, &$ctx )
				{
					if( get_option( 'seraph_accel_cacheOp' ) === false )
						return( false );

					$path = $path . '/' . $item;
					if( @is_dir( $path ) )
						@rmdir( $path );
				}
				, true
			) === false )
			{
				return( false );
			}
		}
	}

	$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );
	if( ( $op == 0 || $op == 2 ) && (isset($settCache[ 'autoProc' ])?$settCache[ 'autoProc' ]:null) )
	{
		CachePrepareQueueDelete( is_multisite() ? $curSiteId : null );

		$ctx = new AnyObj();
		$ctx -> proc = new CacheUrlProcessor( function(){ return( get_option( 'seraph_accel_cacheOp' ) === false ); }, $curSiteId, $settCache, $priority );
		$ctx -> cb = function( $ctx, $url )
		{
			CacheOpUrl_ParseUrl( $url, $siteAddr, $siteSubId, $path, $query );
			if( $ctx -> proc -> op( $siteAddr, $path, $query ) === false )
				return( false );
		};

		if( Op_DepItems_Process( Gen::GetArrField( $settCache, array( 'updAllDeps' ), array() ), array( $ctx, 'cb' ) ) === false )
			return( false );

		CachePushQueueProcessor();
	}

	_DelExpirableOption( 'seraph_accel_cacheOp' );
	return( true );
}

function _CacheObjFileOp( $objFile, $op )
{
	switch( $op )
	{
	case 0:
		if( Gen::GetFileExt( $objFile ) === 'p' )
			@unlink( $objFile );
		else
			@touch( $objFile, 0 );
		break;

	case 2:
		@unlink( $objFile );
		break;
	}

}

function CacheOpUser( $userId, $op )
{
	$curSiteId = GetSiteId();

	$ctx = array( 'op' => $op );

	_CacheDirWalk( $curSiteId, null, null, $ctx, array( 'userId' => $userId ),
		function( &$ctx, $isUserCtx, $objFile )
		{
			_CacheObjFileOp( $objFile, $ctx[ 'op' ] );
		}
	);

}

function _Op_DepItem_Process_GetFldFromObj( $obj, $fld )
{
	if( is_array( $obj ) )
	{
		foreach( $obj as $objSub )
			if( ( $v = Gen::GetArrField( $objSub, $fld ) ) !== null )
				return( $v );

		return( null );
	}

	return( Gen::GetArrField( $obj, $fld ) );
}

function _Op_DepItem_Process_GetValFromObj( $obj, $v )
{
	if( $v === null || (isset($v[ 0 ])?$v[ 0 ]:null) != '{' )
		return( $v );
	$v = _Op_DepItem_Process_GetFldFromObj( $obj, substr( $v, 1, -1 ) );
	return( is_string( $v ) ? str_replace( array( ':', '<', '|', '>', '@', ',' ), array( '%3A', '%3C', '%7C', '%3E', '%40', '%2C' ), $v ) : $v );
}

function _Op_DepItem_Process_GetValsFromObj( $obj, $vals )
{
	if( !is_string( $vals ) || !strlen( $vals ) )
		return( array() );

	$vals = explode( ',', $vals );
	foreach( $vals as &$val )
		$val = _Op_DepItem_Process_GetValFromObj( $obj, $val );
	return( $vals );
}

function _Op_DepItem_UrlAddPath( $url, $path )
{
	if( !$path )
		return( $url );
	return( rtrim( $url, '/' ) . '/' . ltrim( $path, '/' ) );
}

function _Op_DepItem_UrlAddArgs( $url, $args )
{
	$urlComps = Net::UrlParse( $url, Net::URLPARSE_F_QUERY );
	if( !$urlComps )
		return( null );

	$urlComps[ 'query' ] = array_merge( $urlComps[ 'query' ], $args );
	return( Net::UrlDeParse( $urlComps ) );
}

function _Op_DepItems_ParseEx( $dependItem )
{
	$dependItem = '<' . str_replace( array( '@>', '@<' ), array( '@%3E', '@%3C' ), $dependItem ) . '>';

	for( $i = 0; $i < strlen( $dependItem ); $i++ )
	{
		if( $dependItem[ $i ] == '<' )
		{
			if( $i && $dependItem[ $i - 1 ] != '<' )
			{
				$repl = $dependItem[ $i - 1 ] == '>' ? ',' : '",';
				$dependItem = substr_replace( $dependItem, $repl, $i, 0 );
				$i += strlen( $repl );
			}
		}
		else if( $dependItem[ $i ] == '>' )
		{
			if( $dependItem[ $i - 1 ] != '<' && $dependItem[ $i - 1 ] != '>' )
			{
				$repl = $dependItem[ $i - 1 ] == '|' ? '""' : '"';
				$dependItem = substr_replace( $dependItem, $repl, $i, 0 );
				$i += strlen( $repl );
			}
		}
		else if( $dependItem[ $i ] == '|' )
		{
			if( $dependItem[ $i - 1 ] != '>' )
			{
				$repl = ( $dependItem[ $i - 1 ] == '<' || $dependItem[ $i - 1 ] == '|' ) ? '""' : '"';
				$dependItem = substr_replace( $dependItem, $repl, $i, 0 );
				$i += strlen( $repl );
			}
		}
		else
		{
			$repl = '';
			if( $dependItem[ $i - 1 ] == '<' || $dependItem[ $i - 1 ] == '|' )
				$repl = '"';
			else if( $dependItem[ $i - 1 ] == '>' )
				$repl = ',"';

			if( $repl )
			{
				$dependItem = substr_replace( $dependItem, $repl, $i, 0 );
				$i += strlen( $repl );
			}
		}
	}

	$dependItem = substr( str_replace( array( '<', '|', '>' ), array( '[[', '],[', ']]' ), $dependItem ), 1, -1 );
	return( $dependItem );
}

function _Op_DepItems_PostParse( &$dependItem )
{
	for( $i = 0; $i < count( $dependItem ); $i++ )
	{
		if( is_array( $dependItem[ $i ] ) )
		{
			foreach( $dependItem[ $i ] as &$dependItemSub )
				_Op_DepItems_PostParse( $dependItemSub );
			continue;
		}

		$d = explode( ':', trim( $dependItem[ $i ], ':' ) );
		foreach( $d as &$di )
		{
			$di = new AnyObj( array( 'args' => explode( '@', $di ) ) );
			if( count( $di -> args ) > 1 && $di -> args[ 0 ] === '' )
			{
				array_shift( $di -> args );
				$di -> name = array_shift( $di -> args );
			}
		}

		array_splice( $dependItem, $i, 1, $d );
		$i += count( $d ) - 1;
	}
}

function _Op_DepItems_Parse( $dependItem )
{
	$dependItem = @json_decode( _Op_DepItems_ParseEx( $dependItem ), true );
	_Op_DepItems_PostParse( $dependItem );
	return( $dependItem );
}

function _Op_DepItem_Process( $dependItem, $cb, $obj = null, $url = '' )
{
	if( !$dependItem )
		return( @call_user_func( $cb, $url ) );

	$p = array_shift( $dependItem );
	if( is_array( $p ) )
	{
		foreach( $p as $pi )
			if( _Op_DepItem_Process( array_merge( $pi, $dependItem ), $cb, $obj, $url ) === false )
				return( false );
		return( null );
	}

	if( (isset($p -> name)?$p -> name:null) === null )
		return( _Op_DepItem_Process( $dependItem, $cb, $obj, $url . (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) ) );

	switch( $p -> name )
	{
	case 'IF':
		$val = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		if( count( $p -> args ) > 2 )
		{
			$vOp = (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null);
			$valsCmp = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 2 ])?$p -> args[ 2 ]:null) );
		}
		else
		{
			$vOp = '=';
			$valsCmp = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) );
		}

		switch( $vOp )
		{
		case '=':				$isTrue = in_array( $val, $valsCmp ); break;
		case '!=':				$isTrue = !in_array( $val, $valsCmp ); break;
		case '%3C':	$isTrue = $val <  (isset($valsCmp[ 0 ])?$valsCmp[ 0 ]:null); break;
		case '%3C=':	$isTrue = $val <= (isset($valsCmp[ 0 ])?$valsCmp[ 0 ]:null); break;
		case '%3E':	$isTrue = $val >  (isset($valsCmp[ 0 ])?$valsCmp[ 0 ]:null); break;
		case '%3E=':	$isTrue = $val >= (isset($valsCmp[ 0 ])?$valsCmp[ 0 ]:null); break;
		}

		if( !is_array( (isset($dependItem[ 0 ])?$dependItem[ 0 ]:null) ) )
			return( $isTrue ? _Op_DepItem_Process( $dependItem, $cb, $obj, $url ) : null );

		$pi = array_shift( $dependItem );
		$pi = (isset($pi[ $isTrue ? 0 : 1 ])?$pi[ $isTrue ? 0 : 1 ]:null);
		if( !is_array( $pi ) )
			$pi = array();
		return( _Op_DepItem_Process( array_merge( $pi, $dependItem ), $cb, $obj, $url ) );

	case 'home':
		return( _Op_DepItem_Process( $dependItem, $cb, $obj, home_url() ) );

	case 'path':
		$val = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		$urlComps = Net::UrlParse( $url );
		if( !$urlComps )
			return( null );

		$urlComps[ 'path' ] = _Op_DepItem_UrlAddPath( $urlComps[ 'path' ], $val );
		return( _Op_DepItem_Process( $dependItem, $cb, $obj, Net::UrlDeParse( $urlComps ) ) );

	case 'arg':
		return( _Op_DepItem_Process( $dependItem, $cb, $obj, _Op_DepItem_UrlAddArgs( $url, array( rawurldecode( _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) ) ) => rawurldecode( _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) ) ) ) ) ) );

	case 'post':
		foreach( _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) ) as $postId )
		{
			$post = get_post( intval( $postId ) );
			if( _Op_DepItem_Process( $dependItem, $cb, $post, get_permalink( $post ) ) === false )
				return( false );
		}

		return( null );

	case 'posts':
		$postTypes = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		if( !$postTypes )
			return( null );

		$nMaxItems = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) );
		if( $nMaxItems !== null )
		{
			$nMaxItems = intval( $nMaxItems );
			if( $nMaxItems <= 0 )
				$nMaxItems = 1;
		}

		for( $i = 0; ; $i++ )
		{
			$items = DbTbl::GetRows( Db::GetSysTbl( 'posts' ), array( 'ID', 'post_type' ), array( $i * 1000, 1000 ), array( 'post_type' => $postTypes, 'post_status' => 'publish' ), array( 'ID' => 'ASC' ), OBJECT );
			if( !$items )
				break;

			foreach( $items as $item )
			{
				if( _Op_DepItem_Process( $dependItem, $cb, $item, get_permalink( $item -> ID ) ) === false )
					return( false );

				if( $nMaxItems !== null )
				{
					$nMaxItems--;
					if( !$nMaxItems )
						break;
				}
			}
		}

		return( null );

	case 'postsWithTerms':
		$postTypes = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		if( !$postTypes )
			return( null );

		$termIds = array_map( 'intval', _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) ) );

		$nMaxItems = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 2 ])?$p -> args[ 2 ]:null) );
		if( $nMaxItems !== null )
		{
			$nMaxItems = intval( $nMaxItems );
			if( $nMaxItems <= 0 )
				$nMaxItems = 1;
		}

		$tblPosts = Db::GetSysTbl( 'posts' );
		$tblTermRels = Db::GetSysTbl( 'term_relationships' );

		for( $i = 0; ; $i++ )
		{
			$items = DbTbl::GetRowsEx( DbTbl::QueryId( $tblPosts ) . ' LEFT JOIN ' . DbTbl::QueryId( $tblTermRels ) . ' ON (' . DbTbl::QueryId( $tblPosts . '.ID' ) . ' = ' . DbTbl::QueryId( $tblTermRels . '.object_id' ) . ')', array( 'ID', 'post_type' ), array( $i * 1000, 1000 ), array( $tblTermRels . '.term_taxonomy_id' => $termIds, $tblPosts . '.post_type' => $postTypes, $tblPosts . '.post_status' => 'publish' ), array( 'ID' => 'ASC' ), OBJECT, array( 'group' => array( $tblPosts . '.ID' ) ) );
			if( !$items )
				break;

			foreach( $items as $item )
			{
				if( _Op_DepItem_Process( $dependItem, $cb, $item, get_permalink( $item -> ID ) ) === false )
					return( false );

				if( $nMaxItems !== null )
				{
					$nMaxItems--;
					if( !$nMaxItems )
						break;
				}
			}
		}

		return( null );

	case 'terms':
		$taxonomies = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		$postId = intval( _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) ) );
		$nMaxItems = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 2 ])?$p -> args[ 2 ]:null) );
		$flags = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 3 ])?$p -> args[ 3 ]:null) );

		if( $nMaxItems !== null )
		{
			$nMaxItems = intval( $nMaxItems );
			if( $nMaxItems <= 0 )
				$nMaxItems = 1;
		}

		if( $postId )
		{
			foreach( Gen::GetArrField( wp_get_object_terms( $postId, $taxonomies, array( 'fields' => 'all' ) ), array( '' ), array() ) as $term )
				if( _Op_DepItem_Process( $dependItem, $cb, array( $term, $obj ), get_term_link( $term ) ) === false )
					return( false );
		}
		else
		{
			$extraWhere = array();
			if( in_array( 'hideNonEmpty', $flags ) )
				$extraWhere[] = '`count` = 0';
			else if( in_array( 'hideEmpty', $flags ) )
				$extraWhere[] = '`count` > 0';

			for( $i = 0; ; $i++ )
			{
				$nMaxItemsCur = 1000;
				if( $nMaxItems !== null && $nMaxItemsCur > $nMaxItems )
					$nMaxItemsCur = $nMaxItems;
				$terms = DbTbl::GetRows( Db::GetSysTbl( 'term_taxonomy' ), array( 'term_id', 'taxonomy' ), array( $i * 1000, $nMaxItemsCur ), array( 'taxonomy' => $taxonomies ), array( 'term_id' => 'ASC' ), OBJECT, array( 'extraWhere' => $extraWhere ) );
				if( !$terms )
					break;

				foreach( $terms as $term )
				{
					$term -> term_id = intval( $term -> term_id );
					if( _Op_DepItem_Process( $dependItem, $cb, $term, get_term_link( $term -> term_id ) ) === false )
						return( false );
				}

				if( $nMaxItems !== null )
				{
					$nMaxItems -= count( $terms );
					if( $nMaxItems <= 0 )
						break;
				}
			}

		}

		return( null );

	case 'termsOfClass':
		$taxonomyClasses = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		$postType = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) );
		$postId = intval( _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 2 ])?$p -> args[ 2 ]:null) ) );

		if( $postType )
		{
			$postTypeObject = get_post_type_object( $postType );
			if( (isset($postTypeObject -> capability_type)?$postTypeObject -> capability_type:null) )
				$postType = $postTypeObject -> capability_type;
			unset( $postTypeObject );
		}

		foreach( $taxonomyClasses as $taxonomyClass )
		{
			if( $postType && $postId )
				foreach( Gen::GetArrField( Wp::GetPostsTaxonomiesByClass( $taxonomyClass, array( 'public' => true, 'hasRewriteSlug' => true, 'postType' => $postType ) ), array( $postType ), array() ) as $taxonomy )
					foreach( Gen::GetArrField( wp_get_object_terms( $postId, $taxonomy, array( 'fields' => 'all' ) ), array( '' ), array() ) as $term )
						if( _Op_DepItem_Process( $dependItem, $cb, array( $term, $obj ), get_term_link( $term ) ) === false )
							return( false );
		}

		return( null );

	case 'postsBase':
		$postTypes = _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );

		foreach( $postTypes as $postType )
		{
			if( function_exists( 'get_post_type_archive_link' ) )
			{
				$link = get_post_type_archive_link( $postType );
				if( $link && _Op_DepItem_Process( $dependItem, $cb, $obj, $link ) === false )
					return( false );
			}
			else if( $postType == 'post' )
			{
				if( _Op_DepItem_Process( $dependItem, $cb, $obj, get_permalink( get_option( 'page_for_posts' ) ) ) === false )
					return( false );
			}
		}

		return( null );

	case 'sitemapItems':
		$sitemapUri = _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) );
		if( !$sitemapUri )
			$sitemapUri = 'sitemap.xml';

		if( _Op_DepItem_EnumSitemapUrls( $dependItem, $cb, _Op_DepItem_UrlAddPath( Wp::GetSiteRootUrl(), $sitemapUri ), $sitemapUri ) === false )
			return( false );

		return( null );

	case 'wooProdVars':
		$product = function_exists( 'wc_get_product' ) ? wc_get_product( intval( _Op_DepItem_Process_GetValFromObj( $obj, (isset($p -> args[ 0 ])?$p -> args[ 0 ]:null) ) ) ) : null;
		if( !$product || !is_a( $product, 'WC_Product_Variable' ) )
			return( null );

		$attrIdsIncl = array_map( 'strtolower', _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 1 ])?$p -> args[ 1 ]:null) ) );
		$attrIdsExcl = array_map( 'strtolower', _Op_DepItem_Process_GetValsFromObj( $obj, (isset($p -> args[ 2 ])?$p -> args[ 2 ]:null) ) );

		$variationsUnique = array();
		$variations = $product -> get_available_variations();
		foreach( $variations as $variation )
		{
			$attrs = Gen::GetArrField( $variation, array( 'attributes' ), array() );
			$variationUniqueKey = '';
			foreach( $attrs as $attrArg => $attrVal )
			{
				$attrId = substr( $attrArg, 10 );
				if( ( $attrIdsIncl && !in_array( $attrId, $attrIdsIncl ) ) || ( $attrIdsExcl && in_array( $attrId, $attrIdsExcl ) ) )
					unset( $attrs[ $attrArg ] );
				else
					$variationUniqueKey .= $attrId . $attrVal;
			}

			if( !(isset($variationsUnique[ $variationUniqueKey ])?$variationsUnique[ $variationUniqueKey ]:null) )
			{
				if( $attrs && _Op_DepItem_Process( $dependItem, $cb, $item, _Op_DepItem_UrlAddArgs( $url, $attrs ) ) === false )
					return( false );
				$variationsUnique[ $variationUniqueKey ] = true;
			}
		}

		unset( $attrIdsIncl );
		unset( $attrIdsExcl );

		return( null );
	}

	return( null );
}

function _Op_DepItem_EnumSitemapUrls( $dependItem, $cb, $urlSitemap, $tryLocalUri = null )
{

	$data = @GetExtContents( $urlSitemap );

	if( !$data && $tryLocalUri )
		$data = @file_get_contents( _Op_DepItem_UrlAddPath( ABSPATH, $tryLocalUri ) );

	if( !$data )
		return( null );

	$doc = new \DOMDocument();
	$doc -> strictErrorChecking = false;

	if( !@$doc -> loadXML( $data, LIBXML_BIGLINES | LIBXML_NONET | LIBXML_PARSEHUGE ) )
		return( null );

	foreach( $doc -> getElementsByTagName( 'sitemap' ) as $child )
		foreach( $child -> getElementsByTagName( 'loc' ) as $loc )
			if( _Op_DepItem_EnumSitemapUrls( $dependItem, $cb, $loc -> nodeValue ) === false )
				return( false );

	foreach( $doc -> getElementsByTagName( 'url' ) as $child )
		foreach( $child -> getElementsByTagName( 'loc' ) as $loc )
			if( $loc -> tagName == 'loc' && _Op_DepItem_Process( $dependItem, $cb, null, $loc -> nodeValue ) === false )
				return( false );

	return( null );
}

function Op_DepItem_Process( $dependItem, $cb, $obj = null )
{
	$url = '';

	$posProto = strpos( $dependItem, '://' );
	if( ( (isset($dependItem[ 0 ])?$dependItem[ 0 ]:null) === '/' && (isset($dependItem[ 1 ])?$dependItem[ 1 ]:null) === '/' ) || ( $posProto !== false && $posProto <= 5 ) )
	{
		$url = $dependItem;
		$dependItem = '';
	}

	return( _Op_DepItem_Process( _Op_DepItems_Parse( $dependItem ), $cb, $obj, $url ) );
}

function Op_DepItems_Process( $dependItems, $cb, $obj = null )
{
	foreach( $dependItems as $dependItem )
		if( Op_DepItem_Process( $dependItem, $cb, $obj ) === false )
			return( false );
}

function CacheOp_IsPostVisible( $post )
{
	return( in_array( $post -> post_status, array( 'publish' ) ) );
}

function CacheOpPost( $postId, $del, $priority = 0 )
{
    $post = get_post( $postId );
	if( !$post )
		return;

	$sett = Plugin::SettGet();
	$op = Gen::GetArrField( $sett, array( 'cache', 'updPostOp' ), 0 );

	$ctx = new AnyObj();
	$ctx -> urls = array( get_permalink( $post ) );
	$ctx -> cb = function( $ctx, $url )
	{
		$ctx -> urls[] = $url;
	};

	Op_DepItems_Process( Gen::GetArrField( $sett, array( 'cache', 'updPostDeps' ), array() ), array( $ctx, 'cb' ), $post );

	if( $del && $op !== 2 )
	{
		CacheOpUrls( false, $ctx -> urls[ 0 ], 2, $priority );
		array_splice( $ctx -> urls, 0, 1 );
	}

	CacheOpUrls( false, $ctx -> urls, $op, $priority );
}

function CacheOpCancel()
{
	_DelExpirableOption( 'seraph_accel_cacheOp' );
}

function CacheGetCurOp()
{
	return( get_option( 'seraph_accel_cacheOp' ) );
}

function CacheOpUrl_ParseUrl( $url, &$siteAddr, &$siteSubId, &$path, &$query )
{
	global $seraph_accel_sites;

	if( (isset($url[ 0 ])?$url[ 0 ]:null) === '/' && (isset($url[ 1 ])?$url[ 1 ]:null) === '/' )
	{
	}
	else if( strpos( $url, '://' ) === false )
	{
		if( $url && $url[ 0 ] != '/' )
			$url = '/' . $url;
		$url = rtrim( Wp::GetSiteRootUrl(), '/' ) . $url;
	}

	$urlComps = Net::UrlParse( $url, Net::URLPARSE_F_PATH_FIXFIRSTSLASH | Net::URLPARSE_F_PRESERVEEMPTIES );
	if( !(isset($urlComps[ 'scheme' ])?$urlComps[ 'scheme' ]:null) )
		Net::GetUrlWithoutProtoEx( Wp::GetSiteRootUrl(), $urlComps[ 'scheme' ] );

	$host = Net::GetSiteAddrFromUrl( Net::UrlDeParse( $urlComps, 0, array(), array( PHP_URL_SCHEME, PHP_URL_HOST, PHP_URL_PORT ) ) );
	$path = CachePathNormalize( $urlComps[ 'path' ], $pathIsDir );
	$query = (isset($urlComps[ 'query' ])?$urlComps[ 'query' ]:null);
	$siteId = GetCacheSiteIdAdjustPath( $seraph_accel_sites, $host, $siteSubId, $path );
	$siteAddr = $urlComps[ 'scheme' ] . '://' . $host;

	if( $pathIsDir )
		$path .= '/';
	if( $path && $path[ 0 ] != '/' )
		$path = '/' . $path;

	return( $siteId );
}

function CacheOpUrl_DeParseUrl( $siteAddr, $path, $query = null )
{
	$url = $siteAddr . $path;
	if( $query !== null )
		$url .= '?' . $query;
	return( $url );
}

function CacheOpUrls( $isExpr, $urls, $op, $priority = 0, $global = true, $proc = null, $viewId = null )
{
	if( $global && get_option( 'seraph_accel_cacheOp' ) !== false )
		return( false );

	if( !is_array( $urls ) )
		$urls = array( $urls );

	if( $global )
		_SetExpirableOption( 'seraph_accel_cacheOp', $op, 1800 );

	$optDropCache = $global ? new WpOptionNonAutoloadDropCache( 'seraph_accel_cacheOp', 5 ) : null;

	$ctx = new AnyObj();
	$ctx -> op = $op;
	$ctx -> curSiteId = GetSiteId();
	$ctx -> curSiteAddr = null;
	$ctx -> curQuery = null;
	$ctx -> global = $global;
	$ctx -> priority = $priority;

	$settCache = Gen::GetArrField( Plugin::SettGet(), array( 'cache' ), array() );

	if( $op !== 1 && ( $proc !== null ? $proc : (isset($settCache[ 'autoProc' ])?$settCache[ 'autoProc' ]:null) ) )
		$ctx -> proc = new CacheUrlProcessor( $global ? function(){ return( get_option( 'seraph_accel_cacheOp' ) === false ); } : null, $ctx -> curSiteId, $settCache, $priority, $viewId );

	$ctx -> cbUrlOp =
		function( $ctx, $url )
		{
			if( CacheOpUrl_ParseUrl( $url, $ctx -> curSiteAddr, $siteSubId, $path, $ctx -> curQuery ) !== $ctx -> curSiteId )
				return;

			$recurse = false;
			if( substr( $path, -1, 1 ) == '*' )
			{
				$path = substr( $path, 0, -1 );
				$recurse = true;
			}

			if( _CacheDirWalk( $ctx -> curSiteId, $siteSubId, $viewId, $ctx, array( 'objPath' => strtolower( trim( $path, '/' ) ), 'objPathRecurse' => $recurse ),
				function( &$ctx, $isUserCtx, $objFile )
				{
					if( $ctx -> global && get_option( 'seraph_accel_cacheOp' ) === false )
						return( false );

					if( $ctx -> op == 2 && !( $ctx -> priority == -500 || $ctx -> priority == -480 ) )
						if( $dsc = CacheReadDsc( $objFile ) )
							if( isset( $dsc[ 'l' ] ) )
								Learn_Clear( $ctx -> curViewDir . '/l/' . Learn_Id2File( $dsc[ 'l' ] ) );

					_CacheObjFileOp( $objFile, $ctx -> op );
				}

				, null

				, $recurse && (isset($ctx -> proc)?$ctx -> proc:null) ? function( &$ctx, $isUserCtx, $siteRelPath )
				{

					if( $ctx -> proc -> op( $ctx -> curSiteAddr, $siteRelPath, $ctx -> curQuery ) === false )
						return( false );
				} : null

				,
				function( &$ctx, $viewDir, $begin )
				{
					if( !$begin )
						return;

					$ctx -> curViewDir = $viewDir;
				}

			) === false )
			{
				return( false );
			}

			if( (isset($ctx -> proc)?$ctx -> proc:null) )
			{
				if( $ctx -> proc -> op( $ctx -> curSiteAddr, $path, $ctx -> curQuery ) === false )
					return( false );
			}

		};

	foreach( $urls as $url )
		if( ( $isExpr ? Op_DepItem_Process( $url, array( $ctx, 'cbUrlOp' ) ) : @call_user_func( array( $ctx, 'cbUrlOp' ), $url ) ) === false )
			return( false );

	if( (isset($ctx -> proc)?$ctx -> proc:null) )
		CachePushQueueProcessor();

	if( $global )
		_DelExpirableOption( 'seraph_accel_cacheOp' );

	return( true );
}

function CacheOpGetViewsHeaders( $settCache, $viewId = null )
{
	$res = array();

	if( $viewId === null || $viewId === 'cmn' )
		$res[] = array( 'User-Agent' => 'Mozilla/99999.9 AppleWebKit/9999999.99 (KHTML, like Gecko) Chrome/999999.0.9999.99 Safari/9999999.99 seraph-accel-Agent/2.15.1' );

	if( !(isset($settCache[ 'views' ])?$settCache[ 'views' ]:null) )
		return( $res );

	$viewsDeviceGrps = Gen::GetArrField( $settCache, array( 'viewsDeviceGrps' ), array() );
	foreach( $viewsDeviceGrps as $viewsDeviceGrp )
	{
		if( !(isset($viewsDeviceGrp[ 'enable' ])?$viewsDeviceGrp[ 'enable' ]:null) )
			continue;
		if( $viewId !== null && $viewId !== (isset($viewsDeviceGrp[ 'id' ])?$viewsDeviceGrp[ 'id' ]:null) )
			continue;

		if( $agents = Gen::GetArrField( $viewsDeviceGrp, array( 'agents' ), array() ) )
			$res[] = array( 'User-Agent' => 'Mozilla/99999.9 AppleWebKit/9999999.99 (KHTML, like Gecko) Chrome/999999.0.9999.99 Safari/9999999.99 seraph-accel-Agent/2.15.1 ' . ucwords( implode( ' ', $agents ) ) );

	}

	return( $res );
}

function CacheVerifyEnvDropin( $sett )
{

	return( @file_get_contents( WP_CONTENT_DIR . '/advanced-cache.php' ) == GetAdvCacheFileContent( $sett ) );

}

function CacheInitEnvDropin( $sett, $init = true )
{
	$file = WP_CONTENT_DIR . '/advanced-cache.php';
	$cont = @file_get_contents( $file );

	if( !$init )
	{
		if( $cont && strpos( $cont, '/* seraphinite-accelerator */' ) !== false )
		{
			@file_put_contents( $file, '' );
			if( function_exists( 'opcache_invalidate' ) )
				opcache_invalidate( $file, true );
		}

		return( Gen::S_OK );
	}

	$contNew = GetAdvCacheFileContent( $sett );

	$hr = Gen::S_OK;
	if( $cont != $contNew )
	{
		$hr = Gen::HrAccom( $hr, @file_put_contents( $file, $contNew ) !== false ? Gen::S_OK : Gen::E_FAIL );
		if( function_exists( 'opcache_invalidate' ) )
			opcache_invalidate( $file, true );
	}

	return( $hr );
}

function IsWpCacheActive()
{
	return( defined( 'WP_CACHE' ) && WP_CACHE );
}

function CacheInitEnv( $sett, $init = true )
{
	$cacheEnable = Gen::GetArrField( $sett, 'cache/enable', true, '/' );

	if( !$cacheEnable || !$init )
	{
		CacheInitEnvDropin( $sett, false );
		wp_clear_scheduled_hook( 'seraph_accel_cacheClearPeriodically' );
	}

	if( !$init )
	{
		if( Gen::HtAccess_IsSupported() )
			Gen::HtAccess_SetBlock( 'seraphinite-accelerator', '' );

		return( Gen::S_OK );
	}

	if( $cacheEnable )
	{
		$hr = Gen::S_OK;
		$hr = Gen::HrAccom( $hr, CacheInitEnvDropin( $sett ) );
		if( !IsWpCacheActive() )
		{
			$hr = Gen::HrAccom( $hr, Php::File_SetDefineVal( Wp::GetConfigFilePath(), 'WP_CACHE', true ) );
			if( Gen::HrSucc( $hr ) )
			    @define( 'WP_CACHE', true );
		}

		$hr = Gen::HrAccom( $hr, Gen::MakeDir( GetCacheDir(), true ) );
	}

	if( Gen::HtAccess_IsSupported() )
	{
		$htaccessBlock = '';

		if( Gen::GetArrField( $sett, 'cacheBr/enable', false, '/' ) )
		{
			$tmStr = '"access plus ' . Gen::GetArrField( $sett, 'cacheBr/timeout', 0, '/' ) . ' minutes"';

			$htaccessBlock .=
				'<IfModule mod_mime.c>' . "\n\t" . 'AddType image/avif .avif' . "\n\t" . 'AddType image/webp .webp' . "\n\t" . 'AddType application/font-woff2 .woff2' . "\n\t" . 'AddType application/x-font-opentype .otf' . "\n" .
				'</IfModule>' . "\n" .
				'<IfModule mod_expires.c>' . "\n\t" . 'ExpiresActive on' . "\n\t" . 'ExpiresByType text/css                      ' . $tmStr . "\n\t" . 'ExpiresByType text/javascript               ' . $tmStr . "\n\t" . 'ExpiresByType application/javascript        ' . $tmStr . "\n\t" . 'ExpiresByType application/x-javascript      ' . $tmStr . "\n\t" . 'ExpiresByType font/eot                      ' . $tmStr . "\n\t" . 'ExpiresByType font/opentype                 ' . $tmStr . "\n\t" . 'ExpiresByType font/woff                     ' . $tmStr . "\n\t" . 'ExpiresByType application/vnd.ms-fontobject ' . $tmStr . "\n\t" . 'ExpiresByType application/font-woff         ' . $tmStr . "\n\t" . 'ExpiresByType application/font-woff2        ' . $tmStr . "\n\t" . 'ExpiresByType application/x-font-ttf        ' . $tmStr . "\n\t" . 'ExpiresByType application/x-font-woff       ' . $tmStr . "\n\t" . 'ExpiresByType image/vnd.microsoft.icon      ' . $tmStr . "\n\t" . 'ExpiresByType image/x-icon                  ' . $tmStr . "\n\t" . 'ExpiresByType image/bmp                     ' . $tmStr . "\n\t" . 'ExpiresByType image/gif                     ' . $tmStr . "\n\t" . 'ExpiresByType image/jpeg                    ' . $tmStr . "\n\t" . 'ExpiresByType image/png                     ' . $tmStr . "\n\t" . 'ExpiresByType image/svg+xml                 ' . $tmStr . "\n\t" . 'ExpiresByType image/avif                    ' . $tmStr . "\n\t" . 'ExpiresByType image/webp                    ' . $tmStr . "\n\t" . 'ExpiresByType audio/ogg                     ' . $tmStr . "\n\t" . 'ExpiresByType video/mp4                     ' . $tmStr . "\n\t" . 'ExpiresByType video/ogg                     ' . $tmStr . "\n\t" . 'ExpiresByType video/webm                    ' . $tmStr . "\n" .
				'</IfModule>' . "\n" .

				'<IfModule mod_headers.c>' . "\n\t" . 'Header unset ETag' . "\n" .
				'</IfModule>' . "\n" .
				'FileETag None' . "\n" .
				'';
		}

		foreach( array( 'avif', 'webp' ) as $comprType )
		{
			if( !Gen::GetArrField( $sett, array( 'contPr', 'img', $comprType, 'redir' ), false ) )
				continue;

			$htaccessBlock .=
				'<IfModule mod_rewrite.c>' . "\n\t" . 'RewriteEngine On' . "\n\t" . 'RewriteCond %{HTTP_ACCEPT} image\\/' . $comprType . "\n\t" . 'RewriteCond %{REQUEST_FILENAME} \\.(jpe|jpg|jpeg|png|gif|bmp)$' . "\n\t" . 'RewriteCond %{REQUEST_FILENAME}.' . $comprType . ' -f' . "\n\t" . 'RewriteRule ^(.*)\\.(jpe|jpg|jpeg|png|gif|bmp)$ $1\\.$2\\.' . $comprType . ' [QSA]' . "\n" .
				'</IfModule>' . "\n" .
				'<IfModule mod_headers.c>' . "\n\t" . '<FilesMatch \\.(jpe|jpg|jpeg|png|gif|bmp)\\.' . $comprType . '$>' . "\n\t\t" . 'Header merge Vary Accept' . "\n" .
				( Gen::GetArrField( $sett, array( 'hdrTrace' ), false ) ? "\t\t" . 'Header set X-seraph-accel-Cache "2.15.1; state=preoptimized;"' . "\n" : '' ) .
				"\t" . '</FilesMatch>' . "\n" .
				'</IfModule>' . "\n" .
				'';
		}

		{
			$encs = Gen::GetArrField( $sett, array( 'cache', 'encs' ), array() );
			$mimeTypes = array(
				'text/plain',

				'text/css',

				'text/javascript',
				'application/javascript',
				'application/x-javascript',
				'application/json',

				'text/html',
				'text/xml',
				'application/atom+xml',
				'application/rss+xml',
				'application/xhtml+xml',
				'application/xml',
				'text/x-component',

				'application/vnd.ms-fontobject',
				'application/x-font-ttf',
				'font/eot',
				'font/opentype',

				'image/bmp',
				'image/svg+xml',
				'image/vnd.microsoft.icon',
				'image/x-icon',
			);

			if( in_array( 'br', $encs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_brotli.c>' . "\n\t" . '<IfModule mod_filter.c>' . "\n\t\t" . 'AddOutputFilterByType BROTLI_COMPRESS';

				foreach( $mimeTypes as $mimeType )
					$htaccessBlock .= ' ' . $mimeType;

				$htaccessBlock .=
					"\n\t" . '</IfModule>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}

			if( in_array( 'gzip', $encs ) || in_array( 'deflate', $encs ) || in_array( 'compress', $encs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_deflate.c>' . "\n\t" . '<IfModule mod_filter.c>' . "\n\t\t" . 'AddOutputFilterByType DEFLATE';

				foreach( $mimeTypes as $mimeType )
					$htaccessBlock .= ' ' . $mimeType;

				$htaccessBlock .=
					"\n\t" . '</IfModule>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}
		}

		{
			$dataComprs = Gen::GetArrField( $sett, array( 'cache', 'dataCompr' ), array() );

			if( in_array( 'brotli', $dataComprs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_headers.c>' . "\n\t" . '<IfModule mod_rewrite.c>' . "\n\t\t" . 'RewriteEngine On' . "\n\t\t" . 'RewriteCond %{HTTP:Accept-Encoding} (^|\\W)br(\\W|$)' . "\n\t\t" . 'RewriteCond %{REQUEST_FILENAME} \\.(css|js)$' . "\n\t\t" . 'RewriteCond %{REQUEST_FILENAME}.br -f' . "\n\t\t" . 'RewriteRule ^(.*)\\.(css|js)$ $1\\.$2\\.br [QSA]' . "\n\t\t" . 'RewriteRule \\.css\\.br$ - [T=text/css,E=no-gzip:1,E=no-brotli:1]' . "\n\t\t" . 'RewriteRule \\.js\\.br$ - [T=application/javascript,E=no-gzip:1,E=no-brotli:1]' . "\n\t" . '</IfModule>' . "\n\t" . '<FilesMatch \\.(js|css)\\.br$>' . "\n\t\t" . 'Header set Content-Encoding br' . "\n\t\t" . 'Header merge Vary Accept-Encoding' . "\n" .
					( Gen::GetArrField( $sett, array( 'hdrTrace' ), false ) ? "\t\t" . 'Header set X-seraph-accel-Cache "2.15.1; state=precompressed;"' . "\n" : '' ) .
					"\t" . '</FilesMatch>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}

			if( in_array( 'deflate', $dataComprs ) )
			{
				$htaccessBlock .=
					'<IfModule mod_headers.c>' . "\n\t" . '<IfModule mod_rewrite.c>' . "\n\t\t" . 'RewriteEngine On' . "\n\t\t" . 'RewriteCond %{HTTP:Accept-Encoding} (^|\\W)gzip(\\W|$)' . "\n\t\t" . 'RewriteCond %{REQUEST_FILENAME} \\.(css|js)$' . "\n\t\t" . 'RewriteCond %{REQUEST_FILENAME}.gz -f' . "\n\t\t" . 'RewriteRule ^(.*)\\.(css|js)$ $1\\.$2\\.gz [QSA]' . "\n\t\t" . 'RewriteRule \\.css\\.gz$ - [T=text/css,E=no-gzip:1,E=no-brotli:1]' . "\n\t\t" . 'RewriteRule \\.js\\.gz$ - [T=application/javascript,E=no-gzip:1,E=no-brotli:1]' . "\n\t" . '</IfModule>' . "\n\t" . '<FilesMatch \\.(js|css)\\.gz$>' . "\n\t\t" . 'Header set Content-Encoding gzip' . "\n\t\t" . 'Header merge Vary Accept-Encoding' . "\n" .
					( Gen::GetArrField( $sett, array( 'hdrTrace' ), false ) ? "\t\t" . 'Header set X-seraph-accel-Cache "2.15.1; state=precompressed;"' . "\n" : '' ) .
					"\t" . '</FilesMatch>' . "\n" .
					'</IfModule>' . "\n" .
					'';
			}
		}

		$htaccessBlock = trim( $htaccessBlock );

		if( Gen::HtAccess_GetBlock( 'seraphinite-accelerator' ) != $htaccessBlock )
			$hr = Gen::HrAccom( $hr, Gen::HtAccess_SetBlock( 'seraphinite-accelerator', $htaccessBlock, 5 ) );
	}
	else
	{
		$confComprRedirBlock = '';

		foreach( array( 'avif', 'webp' ) as $comprType )
		{
			if( !Gen::GetArrField( $sett, array( 'contPr', 'img', $comprType, 'redir' ), false ) )
				continue;

			if( $confComprRedirBlock )
				$confComprRedirBlock .=
					"\n" .
					'';

			$confComprRedirBlock .=
				"\t" . '# ' . $comprType . '' . "\n\t" . 'set $' . $comprType . '_redir "";' . "\n\t" . 'if ($http_accept ~* "image\\/' . $comprType . '") { set $' . $comprType . '_redir "${' . $comprType . '_redir}A"; }' . "\n\t" . 'if (-f $request_filename.' . $comprType . ') { set $' . $comprType . '_redir "${' . $comprType . '_redir}F"; }' . "\n\t" . 'if ($' . $comprType . '_redir = "AF") { add_header Vary Accept; rewrite (.*) $1.' . $comprType . ' break; }' . "\n" .
				'';
		}

		$fileConfComprRedir = get_home_path() . 'seraph-accel-img-compr-redir.conf';
		if( $confComprRedirBlock )
		{
			$confComprRedirBlock =
				'# seraphinite-accelerator - Automatic redirection to Avif and WebP versions if they exist' . "\n" .
				'location ~ .*\.(jpe|jpg|jpeg|png|gif|bmp)$' . "\n" .
				'{' . "\n" .
				$confComprRedirBlock .
				'}' . "\n" .
				'';

			if( @file_get_contents( $fileConfComprRedir ) !== $confComprRedirBlock )
				@file_put_contents( $fileConfComprRedir, $confComprRedirBlock );
		}
		else
			@unlink( $fileConfComprRedir );
	}

	return( $hr );
}

function GetCacheStatusInfo( $siteId, $cbCancel )
{
	$info = CacheGetInfo( $siteId, $cbCancel );
	if( !$info )
		return( null );

	return( $info );
}

function GetSiteId( $site = null )
{
	if( !is_multisite() )
		return( 'm' );

	if( !$site )
	{
		$site = get_current_site();
		$site = new AnyObj( array( 'blog_id' => get_current_blog_id(), 'site_id' => $site -> site_id ) );
	}

	if( defined( 'SITE_ID_CURRENT_SITE' ) && defined( 'BLOG_ID_CURRENT_SITE' ) && $site -> blog_id == BLOG_ID_CURRENT_SITE && $site -> site_id == SITE_ID_CURRENT_SITE )
		return( 'm' );

	return( '' . $site -> site_id . '_' . $site -> blog_id );
}

function _AddSiteIdSites( &$sitesIds, $addrSite, $siteId, $availablePlugins )
{
	$sitesIds[ $addrSite ] = $siteId;

	$aAddrSite = array();

	if( in_array( 'sitepress-multilingual-cms', $availablePlugins ) )
		foreach( Gen::GetArrField( get_option( 'icl_sitepress_settings' ), array( 'language_domains' ), array() ) as $lang => $domain )
			$aAddrSite[] = rtrim( $domain, '/' );

	if( in_array( 'multiple-domain', $availablePlugins ) )
	{
		foreach( Gen::GetArrField( get_option( 'multiple-domain-domains' ), array( '' ), array() ) as $domain => $opts )
		{
			$domain = rtrim( $domain, '/' );
			$path = trim( Gen::GetArrField( $opts, array( 'base' ), '' ), '/' );
			if( strlen( $path ) )
				$domain .= '/' . $path;
			$aAddrSite[] = $domain;
		}
	}

	foreach( $aAddrSite as $addr )
		if( !isset( $sitesIds[ $addr ] ) )
			$sitesIds[ $addr ] = $siteId . '-' . md5( $addr );

}

function GetAdvCacheFileContent( $sett )
{
	$content = '<?php' . "\n";
	$content .= '/* seraphinite-accelerator */' . "\n";

	$availablePlugins = Plugin::GetAvailablePlugins();

	$sitesIds = array();
	if( Gen::DoesFuncExist( 'get_sites' ) && is_multisite() )
	{
		foreach( get_sites() as $site )
		{
			switch_to_blog( $site -> blog_id );

			$addrSite = strtolower( Net::GetUrlWithoutProto( Gen::SetLastSlash( Wp::GetSiteRootUrl(), false ) ) );
			$siteId = GetSiteId( $site );
			_AddSiteIdSites( $sitesIds, $addrSite, $siteId, $availablePlugins );

			Plugin::SettCacheClear();
			$settSite = Plugin::SettGet();
			$content .= 'function _seraph_accel_siteSettInlineDetach_' . $siteId . '(){ return ' . var_export( $settSite, true ) . '; }' . "\n";

			restore_current_blog();
		}

		$content .= 'function seraph_accel_siteSettInlineDetach($siteId){ $fn = \'_seraph_accel_siteSettInlineDetach_\' . $siteId; return function_exists($fn) ? call_user_func($fn) : null; }' . "\n";
	}
	else
	{
		$content .= 'function seraph_accel_siteSettInlineDetach($siteId){ return ' . var_export( $sett, true ) . '; }' . "\n";
		_AddSiteIdSites( $sitesIds, Net::GetUrlWithoutProto( Gen::SetLastSlash( Wp::GetSiteRootUrl(), false ) ), 'm', $availablePlugins );
	}

	$content .= '$seraph_accel_sites = ' . var_export( $sitesIds, true ) . ';' . "\n";

	$content .= '@include(WP_CONTENT_DIR . \'/plugins/' . Plugin::GetCurBaseName( false ) . '/cache.php\');' . "\n";
	$content .= '?>';
	return( $content );
}

function GetLoadAvg()
{
	$loadavg = explode( ' ', @file_get_contents( '/proc/loadavg' ) );
	$loadavg = $loadavg[ 0 ];
	return( $loadavg ? intval( $loadavg ) : 0 );
}

function UpdateClientSessId( $curUserId, $token = null, $expirationNew = null )
{
	$siteId = GetSiteId();
	$tmCur = Gen::GetCurRequestTime();

	$sessInfo = GetCacheCurUserSession( $siteId );
	$sessId = (isset($sessInfo[ 'sessId' ])?$sessInfo[ 'sessId' ]:null);

	if( $curUserId )
	{
		if( (isset($sessInfo[ 'userSessId' ])?$sessInfo[ 'userSessId' ]:null) != $token || (isset($sessInfo[ 'expiration' ])?$sessInfo[ 'expiration' ]:null) != $expirationNew || (isset($sessInfo[ 'userId' ])?$sessInfo[ 'userId' ]:null) != $curUserId )
		{
			if( Gen::IsEmpty( $sessId ) )
				$sessId = wp_generate_password( 43, false, false );
			SetCacheCurUserSession( $siteId, $sessId, $token, $curUserId, $expirationNew );
		}
	}
	else if( Gen::IsEmpty( $sessId ) )
	{
		$sessId = wp_generate_password( 43, false, false );
		SetCacheCurUserSession( $siteId, $sessId, '0', 0, $tmCur + 12 * HOUR_IN_SECONDS );
	}
	else if( (isset($sessInfo[ 'userId' ])?$sessInfo[ 'userId' ]:null) || (isset($sessInfo[ 'expiration' ])?$sessInfo[ 'expiration' ]:null) < $tmCur )
		SetCacheCurUserSession( $siteId, $sessId, '0', 0, $tmCur + 12 * HOUR_IN_SECONDS );
}

